# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.access_level import AccessLevel
from ..types.access_right_list_response import AccessRightListResponse
from ..types.access_right_response import AccessRightResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawRuleAccessRightsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_access_rights(
        self, rule_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AccessRightListResponse]:
        """
        Get all access rights for a rule. Only owners can view access rights.

        Parameters
        ----------
        rule_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AccessRightListResponse]
            Access rights retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccessRightListResponse,
                    parse_obj_as(
                        type_=AccessRightListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def grant_access(
        self,
        rule_id: str,
        *,
        user_id: str,
        access_level: AccessLevel,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AccessRightResponse]:
        """
        Grant access rights to a user for a rule. Only owners can grant access.

        Parameters
        ----------
        rule_id : str

        user_id : str
            User ID to grant access to

        access_level : AccessLevel
            Access level to grant

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AccessRightResponse]
            Access granted successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access",
            method="POST",
            json={
                "user_id": user_id,
                "access_level": access_level,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccessRightResponse,
                    parse_obj_as(
                        type_=AccessRightResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_access_level(
        self,
        rule_id: str,
        user_id_: str,
        *,
        user_id: str,
        access_level: AccessLevel,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AccessRightResponse]:
        """
        Update the access level for a user on a rule. Only owners can update access levels.

        Parameters
        ----------
        rule_id : str

        user_id_ : str

        user_id : str
            User ID to grant access to

        access_level : AccessLevel
            Access level to grant

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AccessRightResponse]
            Access level updated successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access/{jsonable_encoder(user_id_)}",
            method="PUT",
            json={
                "user_id": user_id,
                "access_level": access_level,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccessRightResponse,
                    parse_obj_as(
                        type_=AccessRightResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def revoke_access(
        self, rule_id: str, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Revoke access rights from a user for a rule. Only owners can revoke access.

        Parameters
        ----------
        rule_id : str

        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access/{jsonable_encoder(user_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRuleAccessRightsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_access_rights(
        self, rule_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AccessRightListResponse]:
        """
        Get all access rights for a rule. Only owners can view access rights.

        Parameters
        ----------
        rule_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AccessRightListResponse]
            Access rights retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccessRightListResponse,
                    parse_obj_as(
                        type_=AccessRightListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def grant_access(
        self,
        rule_id: str,
        *,
        user_id: str,
        access_level: AccessLevel,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AccessRightResponse]:
        """
        Grant access rights to a user for a rule. Only owners can grant access.

        Parameters
        ----------
        rule_id : str

        user_id : str
            User ID to grant access to

        access_level : AccessLevel
            Access level to grant

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AccessRightResponse]
            Access granted successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access",
            method="POST",
            json={
                "user_id": user_id,
                "access_level": access_level,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccessRightResponse,
                    parse_obj_as(
                        type_=AccessRightResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_access_level(
        self,
        rule_id: str,
        user_id_: str,
        *,
        user_id: str,
        access_level: AccessLevel,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AccessRightResponse]:
        """
        Update the access level for a user on a rule. Only owners can update access levels.

        Parameters
        ----------
        rule_id : str

        user_id_ : str

        user_id : str
            User ID to grant access to

        access_level : AccessLevel
            Access level to grant

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AccessRightResponse]
            Access level updated successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access/{jsonable_encoder(user_id_)}",
            method="PUT",
            json={
                "user_id": user_id,
                "access_level": access_level,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccessRightResponse,
                    parse_obj_as(
                        type_=AccessRightResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def revoke_access(
        self, rule_id: str, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Revoke access rights from a user for a rule. Only owners can revoke access.

        Parameters
        ----------
        rule_id : str

        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/rules/{jsonable_encoder(rule_id)}/access/{jsonable_encoder(user_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.composio_sync_response import ComposioSyncResponse
from ..types.composio_sync_status_response import ComposioSyncStatusResponse
from ..types.list_auth_configs_response import ListAuthConfigsResponse
from ..types.update_tool_status_response import UpdateToolStatusResponse
from .raw_client import AsyncRawAdminConnectorsClient, RawAdminConnectorsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AdminConnectorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAdminConnectorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAdminConnectorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAdminConnectorsClient
        """
        return self._raw_client

    def trigger_connector_sync(
        self,
        *,
        force_full_sync: typing.Optional[bool] = None,
        api_key: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ComposioSyncResponse:
        """
        Trigger connector tool catalog sync.

        Syncs available tools into the local database.
        By default, uses incremental sync which only updates changed tools.
        Use force_full_sync=true to update all tools regardless of changes.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Args:
            force_full_sync: If true, skip incremental checks and update all tools

        Returns:
            Response with job_id and sync statistics

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        force_full_sync : typing.Optional[bool]
            Force full sync, ignoring incremental checks and interval

        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComposioSyncResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.admin_connectors.trigger_connector_sync()
        """
        _response = self._raw_client.trigger_connector_sync(
            force_full_sync=force_full_sync, api_key=api_key, request_options=request_options
        )
        return _response.data

    def get_sync_status(
        self, *, api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ComposioSyncStatusResponse:
        """
        Get connector sync status and statistics.

        Returns information about the last sync, next scheduled sync,
        and tool counts.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Returns:
            Response with sync status information

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComposioSyncStatusResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.admin_connectors.get_sync_status()
        """
        _response = self._raw_client.get_sync_status(api_key=api_key, request_options=request_options)
        return _response.data

    def list_auth_configs(
        self, *, api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListAuthConfigsResponse:
        """
        List all connector auth configs in the project.

        This endpoint helps debug auth config issues and verify which configs
        are available for use. Useful for troubleshooting custom branding issues.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Returns:
            List of all auth configs with their IDs, toolkits, and types

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAuthConfigsResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.admin_connectors.list_auth_configs()
        """
        _response = self._raw_client.list_auth_configs(api_key=api_key, request_options=request_options)
        return _response.data

    def update_tool_status(
        self,
        tool_key: str,
        *,
        is_enabled: bool,
        api_key: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateToolStatusResponse:
        """
        Enable or disable a connector tool.

        Controls whether a tool appears in the /v1/personal-connectors/available endpoint.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Args:
            tool_key: The tool key (e.g., "slack", "github", "notion")
            request: Request body with is_enabled boolean

        Returns:
            Updated tool status

        Raises:
            AuthenticationException: If admin API key is missing or invalid
            ResourceNotFoundException: If tool not found

        Parameters
        ----------
        tool_key : str

        is_enabled : bool

        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateToolStatusResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.admin_connectors.update_tool_status(
            tool_key="tool_key",
            is_enabled=True,
        )
        """
        _response = self._raw_client.update_tool_status(
            tool_key, is_enabled=is_enabled, api_key=api_key, request_options=request_options
        )
        return _response.data


class AsyncAdminConnectorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAdminConnectorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAdminConnectorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAdminConnectorsClient
        """
        return self._raw_client

    async def trigger_connector_sync(
        self,
        *,
        force_full_sync: typing.Optional[bool] = None,
        api_key: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ComposioSyncResponse:
        """
        Trigger connector tool catalog sync.

        Syncs available tools into the local database.
        By default, uses incremental sync which only updates changed tools.
        Use force_full_sync=true to update all tools regardless of changes.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Args:
            force_full_sync: If true, skip incremental checks and update all tools

        Returns:
            Response with job_id and sync statistics

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        force_full_sync : typing.Optional[bool]
            Force full sync, ignoring incremental checks and interval

        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComposioSyncResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.admin_connectors.trigger_connector_sync()


        asyncio.run(main())
        """
        _response = await self._raw_client.trigger_connector_sync(
            force_full_sync=force_full_sync, api_key=api_key, request_options=request_options
        )
        return _response.data

    async def get_sync_status(
        self, *, api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ComposioSyncStatusResponse:
        """
        Get connector sync status and statistics.

        Returns information about the last sync, next scheduled sync,
        and tool counts.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Returns:
            Response with sync status information

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComposioSyncStatusResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.admin_connectors.get_sync_status()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_sync_status(api_key=api_key, request_options=request_options)
        return _response.data

    async def list_auth_configs(
        self, *, api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ListAuthConfigsResponse:
        """
        List all connector auth configs in the project.

        This endpoint helps debug auth config issues and verify which configs
        are available for use. Useful for troubleshooting custom branding issues.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Returns:
            List of all auth configs with their IDs, toolkits, and types

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListAuthConfigsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.admin_connectors.list_auth_configs()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_auth_configs(api_key=api_key, request_options=request_options)
        return _response.data

    async def update_tool_status(
        self,
        tool_key: str,
        *,
        is_enabled: bool,
        api_key: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateToolStatusResponse:
        """
        Enable or disable a connector tool.

        Controls whether a tool appears in the /v1/personal-connectors/available endpoint.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Args:
            tool_key: The tool key (e.g., "slack", "github", "notion")
            request: Request body with is_enabled boolean

        Returns:
            Updated tool status

        Raises:
            AuthenticationException: If admin API key is missing or invalid
            ResourceNotFoundException: If tool not found

        Parameters
        ----------
        tool_key : str

        is_enabled : bool

        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateToolStatusResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.admin_connectors.update_tool_status(
                tool_key="tool_key",
                is_enabled=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_tool_status(
            tool_key, is_enabled=is_enabled, api_key=api_key, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CalendarConnectionResponse(UniversalBaseModel):
    """
    Response schema for a calendar connection.
    """

    credential_id: str
    provider: str
    account_email: typing.Optional[str] = None
    is_active: bool
    last_sync_at: typing.Optional[dt.datetime] = None
    event_count: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.change_log_list_response import ChangeLogListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCrewManagementAdminClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def export_csv(
        self,
        organization_id: str,
        *,
        search_term: typing.Optional[str] = OMIT,
        role_id: typing.Optional[str] = OMIT,
        status_id: typing.Optional[str] = OMIT,
        include_deleted: typing.Optional[bool] = OMIT,
        sort_by: typing.Optional[str] = OMIT,
        sort_order: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Any]:
        """
        Export organization members as CSV file.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role
        - Export limited to 10,000 records

        Args:
            organization_id: Organization ID
            request: Export filters
            db: Database session
            auth: Authenticated user

        Returns:
            CSV file as downloadable response

        Raises:
            ValidationException: If export exceeds 10,000 records

        Parameters
        ----------
        organization_id : str
            Organization ID

        search_term : typing.Optional[str]
            Search across full_name, email, and username

        role_id : typing.Optional[str]
            Filter by role ID

        status_id : typing.Optional[str]
            Filter by status ID

        include_deleted : typing.Optional[bool]
            Include soft-deleted members

        sort_by : typing.Optional[str]
            Field to sort by (full_name, email, role_name, status_name, joined_at)

        sort_order : typing.Optional[str]
            Sort order (asc or desc)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/users/export-csv",
            method="POST",
            json={
                "search_term": search_term,
                "role_id": role_id,
                "status_id": status_id,
                "include_deleted": include_deleted,
                "sort_by": sort_by,
                "sort_order": sort_order,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    parse_obj_as(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def export_pdf(
        self,
        organization_id: str,
        *,
        search_term: typing.Optional[str] = OMIT,
        role_id: typing.Optional[str] = OMIT,
        status_id: typing.Optional[str] = OMIT,
        include_deleted: typing.Optional[bool] = OMIT,
        sort_by: typing.Optional[str] = OMIT,
        sort_order: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Any]:
        """
        Export organization members as PDF file.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role
        - Export limited to 10,000 records

        Args:
            organization_id: Organization ID
            request: Export filters
            db: Database session
            auth: Authenticated user

        Returns:
            PDF file as downloadable response

        Raises:
            ValidationException: If export exceeds 10,000 records

        Parameters
        ----------
        organization_id : str
            Organization ID

        search_term : typing.Optional[str]
            Search across full_name, email, and username

        role_id : typing.Optional[str]
            Filter by role ID

        status_id : typing.Optional[str]
            Filter by status ID

        include_deleted : typing.Optional[bool]
            Include soft-deleted members

        sort_by : typing.Optional[str]
            Field to sort by (full_name, email, role_name, status_name, joined_at)

        sort_order : typing.Optional[str]
            Sort order (asc or desc)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/users/export-pdf",
            method="POST",
            json={
                "search_term": search_term,
                "role_id": role_id,
                "status_id": status_id,
                "include_deleted": include_deleted,
                "sort_by": sort_by,
                "sort_order": sort_order,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    parse_obj_as(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_organization_change_logs(
        self,
        organization_id: str,
        *,
        skip: typing.Optional[int] = OMIT,
        take: typing.Optional[int] = OMIT,
        start_date: typing.Optional[dt.datetime] = OMIT,
        end_date: typing.Optional[dt.datetime] = OMIT,
        resource_type: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ChangeLogListResponse]:
        """
        Get paginated change logs for the organization.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role

        Args:
            organization_id: Organization ID
            request: Pagination and filter parameters
            db: Database session
            auth: Authenticated user

        Returns:
            Paginated list of audit logs for the organization

        Parameters
        ----------
        organization_id : str
            Organization ID

        skip : typing.Optional[int]
            Number of records to skip

        take : typing.Optional[int]
            Number of records to return (max 100)

        start_date : typing.Optional[dt.datetime]
            Filter logs from this date

        end_date : typing.Optional[dt.datetime]
            Filter logs until this date

        resource_type : typing.Optional[str]
            Filter by resource type (e.g., 'organization_user', 'assistant', 'rule', 'document')

        action : typing.Optional[str]
            Filter by action (e.g., 'created', 'updated', 'deleted', 'added', 'removed')

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChangeLogListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/dashboard/change-logs",
            method="POST",
            json={
                "skip": skip,
                "take": take,
                "start_date": start_date,
                "end_date": end_date,
                "resource_type": resource_type,
                "action": action,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangeLogListResponse,
                    parse_obj_as(
                        type_=ChangeLogListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_member_change_logs(
        self,
        organization_id: str,
        user_id: str,
        *,
        skip: typing.Optional[int] = OMIT,
        take: typing.Optional[int] = OMIT,
        start_date: typing.Optional[dt.datetime] = OMIT,
        end_date: typing.Optional[dt.datetime] = OMIT,
        resource_type: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ChangeLogListResponse]:
        """
        Get paginated change logs for a specific member.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role
        - Target user must exist and be a member of the organization

        Args:
            organization_id: Organization ID
            user_id: User ID to get change logs for
            request: Pagination and filter parameters
            db: Database session
            auth: Authenticated user

        Returns:
            Paginated list of audit logs for the specific member

        Raises:
            ResourceNotFoundException: If target user is not found in organization

        Parameters
        ----------
        organization_id : str
            Organization ID

        user_id : str
            User ID to get change logs for

        skip : typing.Optional[int]
            Number of records to skip

        take : typing.Optional[int]
            Number of records to return (max 100)

        start_date : typing.Optional[dt.datetime]
            Filter logs from this date

        end_date : typing.Optional[dt.datetime]
            Filter logs until this date

        resource_type : typing.Optional[str]
            Filter by resource type (e.g., 'organization_user', 'assistant', 'rule', 'document')

        action : typing.Optional[str]
            Filter by action (e.g., 'created', 'updated', 'deleted', 'added', 'removed')

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ChangeLogListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/users/{jsonable_encoder(user_id)}/change-logs",
            method="POST",
            json={
                "skip": skip,
                "take": take,
                "start_date": start_date,
                "end_date": end_date,
                "resource_type": resource_type,
                "action": action,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangeLogListResponse,
                    parse_obj_as(
                        type_=ChangeLogListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCrewManagementAdminClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def export_csv(
        self,
        organization_id: str,
        *,
        search_term: typing.Optional[str] = OMIT,
        role_id: typing.Optional[str] = OMIT,
        status_id: typing.Optional[str] = OMIT,
        include_deleted: typing.Optional[bool] = OMIT,
        sort_by: typing.Optional[str] = OMIT,
        sort_order: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Export organization members as CSV file.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role
        - Export limited to 10,000 records

        Args:
            organization_id: Organization ID
            request: Export filters
            db: Database session
            auth: Authenticated user

        Returns:
            CSV file as downloadable response

        Raises:
            ValidationException: If export exceeds 10,000 records

        Parameters
        ----------
        organization_id : str
            Organization ID

        search_term : typing.Optional[str]
            Search across full_name, email, and username

        role_id : typing.Optional[str]
            Filter by role ID

        status_id : typing.Optional[str]
            Filter by status ID

        include_deleted : typing.Optional[bool]
            Include soft-deleted members

        sort_by : typing.Optional[str]
            Field to sort by (full_name, email, role_name, status_name, joined_at)

        sort_order : typing.Optional[str]
            Sort order (asc or desc)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/users/export-csv",
            method="POST",
            json={
                "search_term": search_term,
                "role_id": role_id,
                "status_id": status_id,
                "include_deleted": include_deleted,
                "sort_by": sort_by,
                "sort_order": sort_order,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    parse_obj_as(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def export_pdf(
        self,
        organization_id: str,
        *,
        search_term: typing.Optional[str] = OMIT,
        role_id: typing.Optional[str] = OMIT,
        status_id: typing.Optional[str] = OMIT,
        include_deleted: typing.Optional[bool] = OMIT,
        sort_by: typing.Optional[str] = OMIT,
        sort_order: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Export organization members as PDF file.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role
        - Export limited to 10,000 records

        Args:
            organization_id: Organization ID
            request: Export filters
            db: Database session
            auth: Authenticated user

        Returns:
            PDF file as downloadable response

        Raises:
            ValidationException: If export exceeds 10,000 records

        Parameters
        ----------
        organization_id : str
            Organization ID

        search_term : typing.Optional[str]
            Search across full_name, email, and username

        role_id : typing.Optional[str]
            Filter by role ID

        status_id : typing.Optional[str]
            Filter by status ID

        include_deleted : typing.Optional[bool]
            Include soft-deleted members

        sort_by : typing.Optional[str]
            Field to sort by (full_name, email, role_name, status_name, joined_at)

        sort_order : typing.Optional[str]
            Sort order (asc or desc)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/users/export-pdf",
            method="POST",
            json={
                "search_term": search_term,
                "role_id": role_id,
                "status_id": status_id,
                "include_deleted": include_deleted,
                "sort_by": sort_by,
                "sort_order": sort_order,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    parse_obj_as(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_organization_change_logs(
        self,
        organization_id: str,
        *,
        skip: typing.Optional[int] = OMIT,
        take: typing.Optional[int] = OMIT,
        start_date: typing.Optional[dt.datetime] = OMIT,
        end_date: typing.Optional[dt.datetime] = OMIT,
        resource_type: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ChangeLogListResponse]:
        """
        Get paginated change logs for the organization.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role

        Args:
            organization_id: Organization ID
            request: Pagination and filter parameters
            db: Database session
            auth: Authenticated user

        Returns:
            Paginated list of audit logs for the organization

        Parameters
        ----------
        organization_id : str
            Organization ID

        skip : typing.Optional[int]
            Number of records to skip

        take : typing.Optional[int]
            Number of records to return (max 100)

        start_date : typing.Optional[dt.datetime]
            Filter logs from this date

        end_date : typing.Optional[dt.datetime]
            Filter logs until this date

        resource_type : typing.Optional[str]
            Filter by resource type (e.g., 'organization_user', 'assistant', 'rule', 'document')

        action : typing.Optional[str]
            Filter by action (e.g., 'created', 'updated', 'deleted', 'added', 'removed')

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChangeLogListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/dashboard/change-logs",
            method="POST",
            json={
                "skip": skip,
                "take": take,
                "start_date": start_date,
                "end_date": end_date,
                "resource_type": resource_type,
                "action": action,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangeLogListResponse,
                    parse_obj_as(
                        type_=ChangeLogListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_member_change_logs(
        self,
        organization_id: str,
        user_id: str,
        *,
        skip: typing.Optional[int] = OMIT,
        take: typing.Optional[int] = OMIT,
        start_date: typing.Optional[dt.datetime] = OMIT,
        end_date: typing.Optional[dt.datetime] = OMIT,
        resource_type: typing.Optional[str] = OMIT,
        action: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ChangeLogListResponse]:
        """
        Get paginated change logs for a specific member.

        Requires:
        - User must be authenticated
        - User must be a member of the organization
        - User must have Admin or Owner role
        - Target user must exist and be a member of the organization

        Args:
            organization_id: Organization ID
            user_id: User ID to get change logs for
            request: Pagination and filter parameters
            db: Database session
            auth: Authenticated user

        Returns:
            Paginated list of audit logs for the specific member

        Raises:
            ResourceNotFoundException: If target user is not found in organization

        Parameters
        ----------
        organization_id : str
            Organization ID

        user_id : str
            User ID to get change logs for

        skip : typing.Optional[int]
            Number of records to skip

        take : typing.Optional[int]
            Number of records to return (max 100)

        start_date : typing.Optional[dt.datetime]
            Filter logs from this date

        end_date : typing.Optional[dt.datetime]
            Filter logs until this date

        resource_type : typing.Optional[str]
            Filter by resource type (e.g., 'organization_user', 'assistant', 'rule', 'document')

        action : typing.Optional[str]
            Filter by action (e.g., 'created', 'updated', 'deleted', 'added', 'removed')

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ChangeLogListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/users/{jsonable_encoder(user_id)}/change-logs",
            method="POST",
            json={
                "skip": skip,
                "take": take,
                "start_date": start_date,
                "end_date": end_date,
                "resource_type": resource_type,
                "action": action,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ChangeLogListResponse,
                    parse_obj_as(
                        type_=ChangeLogListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .recent_decision import RecentDecision


class DecisionsResponse(UniversalBaseModel):
    """
    Decision tracking aggregated from intelligence.
    """

    total_decisions: int
    avg_decisions_per_meeting: float
    meetings_with_decisions: int
    meetings_without_decisions: int
    recent_decisions: typing.Optional[typing.List[RecentDecision]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .update_message_response_role import UpdateMessageResponseRole


class UpdateMessageResponse(UniversalBaseModel):
    """
    Response schema for updated message.
    """

    id: str = pydantic.Field()
    """
    Message ID with msg_ prefix
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Creation timestamp
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last update timestamp
    """

    thread_id: str = pydantic.Field()
    """
    Thread ID with thread_ prefix
    """

    role: UpdateMessageResponseRole = pydantic.Field()
    """
    Message role
    """

    content: str = pydantic.Field()
    """
    Message text content
    """

    attachments: typing.Optional[typing.List[typing.Dict[str, typing.Any]]] = pydantic.Field(default=None)
    """
    File attachments
    """

    metadata: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Custom metadata
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.clear_all_caches_response import ClearAllCachesResponse
from ..types.invalidate_limit_cache_response import InvalidateLimitCacheResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAdminCacheClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def invalidate_limit_caches(
        self,
        *,
        organization_id: str,
        api_key: typing.Optional[str] = None,
        api_key_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[InvalidateLimitCacheResponse]:
        """
        Invalidate limit caches for organization and/or API keys.

        Use this endpoint when limits are updated directly in the database
        to ensure the cache reflects the new limits immediately.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Args:
            cache_request: Cache invalidation request with organization_id and optional api_key_ids
            cache_manager: Limit cache manager instance

        Returns:
            Response with success status and count of caches cleared

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        organization_id : str
            Organization ID to invalidate caches for

        api_key : typing.Optional[str]

        api_key_ids : typing.Optional[typing.Sequence[str]]
            Optional list of API key IDs to invalidate per-key caches

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InvalidateLimitCacheResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/admin/cache/invalidate-limits",
            method="POST",
            json={
                "organization_id": organization_id,
                "api_key_ids": api_key_ids,
            },
            headers={
                "content-type": "application/json",
                "api-key": str(api_key) if api_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InvalidateLimitCacheResponse,
                    parse_obj_as(
                        type_=InvalidateLimitCacheResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def clear_all_caches(
        self, *, api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ClearAllCachesResponse]:
        """
        Clear all application caches (distributed + in-memory).

        Parameters
        ----------
        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ClearAllCachesResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/admin/cache/clear-all",
            method="POST",
            headers={
                "api-key": str(api_key) if api_key is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClearAllCachesResponse,
                    parse_obj_as(
                        type_=ClearAllCachesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAdminCacheClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def invalidate_limit_caches(
        self,
        *,
        organization_id: str,
        api_key: typing.Optional[str] = None,
        api_key_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[InvalidateLimitCacheResponse]:
        """
        Invalidate limit caches for organization and/or API keys.

        Use this endpoint when limits are updated directly in the database
        to ensure the cache reflects the new limits immediately.

        Requires:
        - Header: api-key: {ADMIN_API_KEY}

        Args:
            cache_request: Cache invalidation request with organization_id and optional api_key_ids
            cache_manager: Limit cache manager instance

        Returns:
            Response with success status and count of caches cleared

        Raises:
            AuthenticationException: If admin API key is missing or invalid

        Parameters
        ----------
        organization_id : str
            Organization ID to invalidate caches for

        api_key : typing.Optional[str]

        api_key_ids : typing.Optional[typing.Sequence[str]]
            Optional list of API key IDs to invalidate per-key caches

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InvalidateLimitCacheResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/admin/cache/invalidate-limits",
            method="POST",
            json={
                "organization_id": organization_id,
                "api_key_ids": api_key_ids,
            },
            headers={
                "content-type": "application/json",
                "api-key": str(api_key) if api_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InvalidateLimitCacheResponse,
                    parse_obj_as(
                        type_=InvalidateLimitCacheResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def clear_all_caches(
        self, *, api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ClearAllCachesResponse]:
        """
        Clear all application caches (distributed + in-memory).

        Parameters
        ----------
        api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ClearAllCachesResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/admin/cache/clear-all",
            method="POST",
            headers={
                "api-key": str(api_key) if api_key is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClearAllCachesResponse,
                    parse_obj_as(
                        type_=ClearAllCachesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.add_thread_to_space_response import AddThreadToSpaceResponse
from ..types.add_vector_store_to_space_response import AddVectorStoreToSpaceResponse
from ..types.space_list_response import SpaceListResponse
from ..types.space_response import SpaceResponse
from ..types.space_thread_list_response import SpaceThreadListResponse
from ..types.space_vector_store_list_response import SpaceVectorStoreListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSpacesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_spaces(
        self,
        *,
        organization_id: str,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SpaceListResponse]:
        """
        List all spaces user has access to in an organization

        Parameters
        ----------
        organization_id : str
            Organization ID to filter by

        limit : typing.Optional[int]
            Maximum number of results

        cursor : typing.Optional[str]
            Pagination cursor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SpaceListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/spaces/",
            method="GET",
            params={
                "organization_id": organization_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceListResponse,
                    parse_obj_as(
                        type_=SpaceListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_space(
        self,
        *,
        organization_id: str,
        name: str,
        description: typing.Optional[str] = OMIT,
        access_mode: typing.Optional[str] = OMIT,
        access_users: typing.Optional[typing.Sequence[str]] = OMIT,
        access_departments: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_users: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        visible_to_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SpaceResponse]:
        """
        Create a new space with auto-linked vector store for memory management

        Parameters
        ----------
        organization_id : str
            Organization ID

        name : str
            Space name

        description : typing.Optional[str]
            Optional space description

        access_mode : typing.Optional[str]
            Access mode: private, organization, or public

        access_users : typing.Optional[typing.Sequence[str]]
            User IDs with view access

        access_departments : typing.Optional[typing.Sequence[str]]
            Department IDs with view access

        editable_by_users : typing.Optional[typing.Sequence[str]]
            User IDs with edit access (supports wildcard '*')

        editable_by_roles : typing.Optional[typing.Sequence[str]]
            Role IDs with edit access

        visible_to_roles : typing.Optional[typing.Sequence[str]]
            Role IDs with view access

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SpaceResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/spaces/",
            method="POST",
            json={
                "organization_id": organization_id,
                "name": name,
                "description": description,
                "access_mode": access_mode,
                "access_users": access_users,
                "access_departments": access_departments,
                "editable_by_users": editable_by_users,
                "editable_by_roles": editable_by_roles,
                "visible_to_roles": visible_to_roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceResponse,
                    parse_obj_as(
                        type_=SpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_space(
        self, space_id: str, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SpaceResponse]:
        """
        Retrieve space details by ID with access validation

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SpaceResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}",
            method="GET",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceResponse,
                    parse_obj_as(
                        type_=SpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_space(
        self, space_id: str, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Soft delete space (requires owner access)

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}",
            method="DELETE",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_space(
        self,
        space_id: str,
        *,
        organization_id: str,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        access_mode: typing.Optional[str] = OMIT,
        access_users: typing.Optional[typing.Sequence[str]] = OMIT,
        access_departments: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_users: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        visible_to_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SpaceResponse]:
        """
        Update space properties with access validation

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        name : typing.Optional[str]
            New space name

        description : typing.Optional[str]
            New space description

        access_mode : typing.Optional[str]
            New access mode

        access_users : typing.Optional[typing.Sequence[str]]
            New access users list

        access_departments : typing.Optional[typing.Sequence[str]]
            New access departments list

        editable_by_users : typing.Optional[typing.Sequence[str]]
            New editable users list

        editable_by_roles : typing.Optional[typing.Sequence[str]]
            New editable roles list

        visible_to_roles : typing.Optional[typing.Sequence[str]]
            New visible roles list

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SpaceResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}",
            method="PATCH",
            params={
                "organization_id": organization_id,
            },
            json={
                "name": name,
                "description": description,
                "access_mode": access_mode,
                "access_users": access_users,
                "access_departments": access_departments,
                "editable_by_users": editable_by_users,
                "editable_by_roles": editable_by_roles,
                "visible_to_roles": visible_to_roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceResponse,
                    parse_obj_as(
                        type_=SpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def add_thread_to_space(
        self,
        space_id: str,
        thread_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddThreadToSpaceResponse]:
        """
        Associate a thread with a space (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        thread_id : str
            Thread ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddThreadToSpaceResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/threads/{jsonable_encoder(thread_id)}",
            method="POST",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddThreadToSpaceResponse,
                    parse_obj_as(
                        type_=AddThreadToSpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_thread_from_space(
        self,
        space_id: str,
        thread_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Remove thread from space and cleanup vectors (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        thread_id : str
            Thread ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/threads/{jsonable_encoder(thread_id)}",
            method="DELETE",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_space_threads(
        self,
        space_id: str,
        *,
        organization_id: str,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SpaceThreadListResponse]:
        """
        List all threads associated with a space (requires view access)

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        limit : typing.Optional[int]
            Maximum number of threads to return

        cursor : typing.Optional[str]
            Pagination cursor from previous response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SpaceThreadListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/threads",
            method="GET",
            params={
                "organization_id": organization_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceThreadListResponse,
                    parse_obj_as(
                        type_=SpaceThreadListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def add_vector_store_to_space(
        self,
        space_id: str,
        vector_store_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddVectorStoreToSpaceResponse]:
        """
        Attach an additional external vector store to a space (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        vector_store_id : str
            Vector store ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddVectorStoreToSpaceResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/vector-stores/{jsonable_encoder(vector_store_id)}",
            method="POST",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVectorStoreToSpaceResponse,
                    parse_obj_as(
                        type_=AddVectorStoreToSpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_vector_store_from_space(
        self,
        space_id: str,
        vector_store_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Remove an additional vector store from a space (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        vector_store_id : str
            Vector store ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/vector-stores/{jsonable_encoder(vector_store_id)}",
            method="DELETE",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_space_vector_stores(
        self, space_id: str, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SpaceVectorStoreListResponse]:
        """
        List all additional vector stores attached to a space (requires view access)

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SpaceVectorStoreListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/vector-stores",
            method="GET",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceVectorStoreListResponse,
                    parse_obj_as(
                        type_=SpaceVectorStoreListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSpacesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_spaces(
        self,
        *,
        organization_id: str,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SpaceListResponse]:
        """
        List all spaces user has access to in an organization

        Parameters
        ----------
        organization_id : str
            Organization ID to filter by

        limit : typing.Optional[int]
            Maximum number of results

        cursor : typing.Optional[str]
            Pagination cursor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SpaceListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/spaces/",
            method="GET",
            params={
                "organization_id": organization_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceListResponse,
                    parse_obj_as(
                        type_=SpaceListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_space(
        self,
        *,
        organization_id: str,
        name: str,
        description: typing.Optional[str] = OMIT,
        access_mode: typing.Optional[str] = OMIT,
        access_users: typing.Optional[typing.Sequence[str]] = OMIT,
        access_departments: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_users: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        visible_to_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SpaceResponse]:
        """
        Create a new space with auto-linked vector store for memory management

        Parameters
        ----------
        organization_id : str
            Organization ID

        name : str
            Space name

        description : typing.Optional[str]
            Optional space description

        access_mode : typing.Optional[str]
            Access mode: private, organization, or public

        access_users : typing.Optional[typing.Sequence[str]]
            User IDs with view access

        access_departments : typing.Optional[typing.Sequence[str]]
            Department IDs with view access

        editable_by_users : typing.Optional[typing.Sequence[str]]
            User IDs with edit access (supports wildcard '*')

        editable_by_roles : typing.Optional[typing.Sequence[str]]
            Role IDs with edit access

        visible_to_roles : typing.Optional[typing.Sequence[str]]
            Role IDs with view access

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SpaceResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/spaces/",
            method="POST",
            json={
                "organization_id": organization_id,
                "name": name,
                "description": description,
                "access_mode": access_mode,
                "access_users": access_users,
                "access_departments": access_departments,
                "editable_by_users": editable_by_users,
                "editable_by_roles": editable_by_roles,
                "visible_to_roles": visible_to_roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceResponse,
                    parse_obj_as(
                        type_=SpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_space(
        self, space_id: str, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SpaceResponse]:
        """
        Retrieve space details by ID with access validation

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SpaceResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}",
            method="GET",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceResponse,
                    parse_obj_as(
                        type_=SpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_space(
        self, space_id: str, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Soft delete space (requires owner access)

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}",
            method="DELETE",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_space(
        self,
        space_id: str,
        *,
        organization_id: str,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        access_mode: typing.Optional[str] = OMIT,
        access_users: typing.Optional[typing.Sequence[str]] = OMIT,
        access_departments: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_users: typing.Optional[typing.Sequence[str]] = OMIT,
        editable_by_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        visible_to_roles: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SpaceResponse]:
        """
        Update space properties with access validation

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        name : typing.Optional[str]
            New space name

        description : typing.Optional[str]
            New space description

        access_mode : typing.Optional[str]
            New access mode

        access_users : typing.Optional[typing.Sequence[str]]
            New access users list

        access_departments : typing.Optional[typing.Sequence[str]]
            New access departments list

        editable_by_users : typing.Optional[typing.Sequence[str]]
            New editable users list

        editable_by_roles : typing.Optional[typing.Sequence[str]]
            New editable roles list

        visible_to_roles : typing.Optional[typing.Sequence[str]]
            New visible roles list

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SpaceResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}",
            method="PATCH",
            params={
                "organization_id": organization_id,
            },
            json={
                "name": name,
                "description": description,
                "access_mode": access_mode,
                "access_users": access_users,
                "access_departments": access_departments,
                "editable_by_users": editable_by_users,
                "editable_by_roles": editable_by_roles,
                "visible_to_roles": visible_to_roles,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceResponse,
                    parse_obj_as(
                        type_=SpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def add_thread_to_space(
        self,
        space_id: str,
        thread_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddThreadToSpaceResponse]:
        """
        Associate a thread with a space (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        thread_id : str
            Thread ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddThreadToSpaceResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/threads/{jsonable_encoder(thread_id)}",
            method="POST",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddThreadToSpaceResponse,
                    parse_obj_as(
                        type_=AddThreadToSpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_thread_from_space(
        self,
        space_id: str,
        thread_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Remove thread from space and cleanup vectors (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        thread_id : str
            Thread ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/threads/{jsonable_encoder(thread_id)}",
            method="DELETE",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_space_threads(
        self,
        space_id: str,
        *,
        organization_id: str,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SpaceThreadListResponse]:
        """
        List all threads associated with a space (requires view access)

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        limit : typing.Optional[int]
            Maximum number of threads to return

        cursor : typing.Optional[str]
            Pagination cursor from previous response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SpaceThreadListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/threads",
            method="GET",
            params={
                "organization_id": organization_id,
                "limit": limit,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceThreadListResponse,
                    parse_obj_as(
                        type_=SpaceThreadListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def add_vector_store_to_space(
        self,
        space_id: str,
        vector_store_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddVectorStoreToSpaceResponse]:
        """
        Attach an additional external vector store to a space (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        vector_store_id : str
            Vector store ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddVectorStoreToSpaceResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/vector-stores/{jsonable_encoder(vector_store_id)}",
            method="POST",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVectorStoreToSpaceResponse,
                    parse_obj_as(
                        type_=AddVectorStoreToSpaceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_vector_store_from_space(
        self,
        space_id: str,
        vector_store_id: str,
        *,
        organization_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Remove an additional vector store from a space (requires edit access)

        Parameters
        ----------
        space_id : str
            Space ID

        vector_store_id : str
            Vector store ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/vector-stores/{jsonable_encoder(vector_store_id)}",
            method="DELETE",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_space_vector_stores(
        self, space_id: str, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SpaceVectorStoreListResponse]:
        """
        List all additional vector stores attached to a space (requires view access)

        Parameters
        ----------
        space_id : str
            Space ID

        organization_id : str
            Organization ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SpaceVectorStoreListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/spaces/{jsonable_encoder(space_id)}/vector-stores",
            method="GET",
            params={
                "organization_id": organization_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SpaceVectorStoreListResponse,
                    parse_obj_as(
                        type_=SpaceVectorStoreListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
